import {
  Body,
  Controller,
  Get,
  Post,
  Put,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionProgramaService } from './institucion-programa.service';
import { ValidarUsuarioService } from '../validar-usuario/validar-usuario.service';
import { Operador } from '../operador/entity/operador.entity';
import { Usuario } from '../usuario/entity/usuario.entity';
import { IdInstitucionDto } from '../dto/input/id-institucion.dto';
import { CreateProgramaDto } from './dto/input/create.dto';
import { UpdateProgramaDto } from './dto/input/update.dto';
import { MessageOutputDto } from '../dto/output/message.dto';
import { InstitucionProgramaOutputDto } from './dto/output/institucion-programa.dto';
import { InstitucionProgramaMinOutputDto } from './dto/output/institucion-programa-min.dto';
import { ProgramaOutputDto } from './dto/output/programa.dto';

@Controller('institucion-programa')
@ApiTags('institucion-programa')
export class InstitucionProgramaController {
  constructor(
    private institucionProgramaService: InstitucionProgramaService,
    private validarUsuarioService: ValidarUsuarioService,
  ) {}

  @Serealize(MessageOutputDto)
  @Post()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Crear un nuevo programa.' })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Variables que necesita el endpoint.',
    examples: { ejemplo: { value: { programa: '' } } },
  })
  create(@Request() req, @Body() body: CreateProgramaDto) {
    const superAdmin: Operador = req.user.operador;

    this.validarUsuarioService.validarSuperAdmin(superAdmin);
    return this.institucionProgramaService.create(body.programa);
  }

  @Serealize(ProgramaOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Todos los programas.' })
  @ApiBearerAuth('jwt')
  get(@Request() req) {
    const superAdmin: Operador = req.user.operador;

    this.validarUsuarioService.validarSuperAdminAdmin(superAdmin);
    return this.institucionProgramaService.findAll();
  }

  @Serealize(InstitucionProgramaOutputDto)
  @Get('programas')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Todas las instituciones programas de un operador.',
  })
  @ApiBearerAuth('jwt')
  programas(@Request() req) {
    const operador: Operador = req.user.operador;

    this.validarUsuarioService.validarAdminOperador(operador);
    return this.institucionProgramaService.findAllOperador(operador);
  }

  @Serealize(InstitucionProgramaMinOutputDto)
  @Get('programas-mostrar')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Todas las instituciones programas para mostrar.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  programasMostrar(@Request() req, @Query() query: IdInstitucionDto) {
    const usuario: Usuario = req.user.usuario;

    this.validarUsuarioService.validarUsuario(usuario);
    return this.institucionProgramaService.findAllMostrar(
      usuario,
      parseInt(query.id_institucion),
    );
  }

  @Serealize(MessageOutputDto)
  @Put()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Actualizar la información de una institución programa.',
  })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Variables que necesita el endpoint.',
    examples: {
      ejemplo: { value: { id_institucion_programa: 130, mostrar: true } },
    },
  })
  update(@Request() req, @Body() body: UpdateProgramaDto) {
    const admin: Operador = req.user.operador;

    this.validarUsuarioService.validarAdmin(admin);
    return this.institucionProgramaService.update(admin, body);
  }
}
