import {
  Body,
  Controller,
  Get,
  Post,
  Put,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionTipoEntradaService } from './institucion-tipo-entrada.service';
import { ValidarUsuarioService } from '../validar-usuario/validar-usuario.service';
import { Operador } from '../operador/entity/operador.entity';
import { Usuario } from '../usuario/entity/usuario.entity';
import { IdInstitucionDto } from '../dto/input/id-institucion.dto';
import { CreateInstitucionTipoEntradaDto } from './dto/input/create.dto';
import { UpdateInstitucionTipoEntradaDto } from './dto/input/update.dto';
import { MessageOutputDto } from '../dto/output/message.dto';
import { InstitucionTipoEntradaOutputDto } from './dto/output/institucion-tipo-entrada.dto';
import { InstitucionTipoEntradaMinOutputDto } from './dto/output/institucion-tipo-entrada-min.dto';
import { TipoEntradaOutputDto } from './dto/output/tipo-entrada.dto';

@Controller('institucion-tipo-entrada')
@ApiTags('institucion-tipo-entrada')
export class InstitucionTipoEntradaController {
  constructor(
    private institucionTipoEntradaService: InstitucionTipoEntradaService,
    private validarUsuarioService: ValidarUsuarioService,
  ) {}

  @Serealize(MessageOutputDto)
  @Post()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Crear un nuevo tipo de entrada.' })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Variables que necesita el endpoint.',
    examples: { ejemplo: { value: { tipo_entrada: '' } } },
  })
  create(@Request() req, @Body() body: CreateInstitucionTipoEntradaDto) {
    const superAdmin: Operador = req.user.operador;

    this.validarUsuarioService.validarSuperAdmin(superAdmin);
    return this.institucionTipoEntradaService.create(body.tipo_entrada);
  }

  @Serealize(TipoEntradaOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Todos los tipos de entrada.' })
  @ApiBearerAuth('jwt')
  get(@Request() req) {
    const superAdmin: Operador = req.user.operador;

    this.validarUsuarioService.validarSuperAdminAdmin(superAdmin);
    return this.institucionTipoEntradaService.findAll();
  }

  @Serealize(InstitucionTipoEntradaOutputDto)
  @Get('tipos-entrada')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Todas las instituciones tipo entrada de un operador.',
  })
  @ApiBearerAuth('jwt')
  tiposEntrada(@Request() req) {
    const operador: Operador = req.user.operador;

    this.validarUsuarioService.validarAdminOperador(operador);
    return this.institucionTipoEntradaService.findAllOperador(operador);
  }

  @Serealize(InstitucionTipoEntradaMinOutputDto)
  @Get('tipos-entrada-mostrar')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Todas las instituciones tipo entrada para mostrar.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id de la institución',
    name: 'id_institucion',
    type: 'string',
  })
  tipoEntradaMostrar(@Request() req, @Query() query: IdInstitucionDto) {
    const usuario: Usuario = req.user.usuario;

    this.validarUsuarioService.validarUsuario(usuario);
    return this.institucionTipoEntradaService.findAllMostrar(
      usuario,
      parseInt(query.id_institucion),
    );
  }

  @Serealize(MessageOutputDto)
  @Put()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Actualizar la información de una institución tipo entrada.',
  })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Variables que necesita el endpoint.',
    examples: {
      ejemplo: { value: { id_institucion_tipo_entrada: 130, mostrar: true } },
    },
  })
  update(@Request() req, @Body() body: UpdateInstitucionTipoEntradaDto) {
    const admin: Operador = req.user.operador;

    this.validarUsuarioService.validarAdmin(admin);
    return this.institucionTipoEntradaService.update(admin, body);
  }
}
